CREATE OR REPLACE PACKAGE IBDB.PKG_GESTIO AS

  /** Paquet que conte les funcions de gesti.
  *
  *  @author Juanjo Daz Prez
  *  @version 1.0 Mar-Juny 2013
  **/

  /** Funci que dona de alta un jugador assegurant la seva integritat.
  *
  *  Funci auditada per PKG_UTILS.PR_AUDITAR_CANVIS <BR>
  *
  *  La funci garantitza la segent integritat <BR>
  *   .- El numero que proveir la federaci, aquest nmero no te perqu se nic entre jugadors de diferents pasos, per si que ho haur de ser entre jugadors del mateix pas.
  *   .- Noms es permetr guardar si el jugador est de baixa mdica o baixa per motius personals en cas de que el seu estat sigui de baixa.
  *   .- Els valors de alada i pes no poden ser negatius.
  *  <p>
  *  S'ha de tenir en compte que els segents triggers es veuen utilitzats durant la crida d'aquesta funci:
  *    TRG_AUDIT_JUGADOR <BR>
  *  @param p_jugador Tota la informaci possible seguint la estructura de la taula JUGADOR.
  *  @return PKG_UTILS.C_OK: Si tot s'ha executat correctament <BR>
  *          PKG_UTILS.C_PK_ERROR: Error per violaci de la clau primaria <BR>
  *          PKG_UTILS.C_FK_ERROR: Error per violaci de la clau externa <BR>
  *          PKG_UTILS.C_UNKNOWN_KO: Error desconegut, veure registre en la taula AUDITORIA. <BR>
  *  @author Juanjo Daz Prez
  *  @version 1.0 Mar-Juny 2013
  **/
  FUNCTION FN_ALTA_JUGADOR( p_jugador IBDB.JUGADOR%ROWTYPE) RETURN INTEGER;

  /** Funci que dona de baix un jugador.
  *
  *  Funci auditada per PKG_UTILS.PR_AUDITAR_CANVIS
  *
  *  Al donar de baixa un jugador existent (eliminar-lo) la informaci relacionada a aquest jugador tamb seliminara.
  *    CONTRACTE <BR>
  *    ESTADISTICA_PARTIT <BR>
  *    DIAGNOSTIC <BR>
  *  S'ha de tenir en compte que els segents triggers es veuen utilitzats durant la crida d'aquesta funci:
  *    TRG_AUDIT_JUGADOR <BR>
  *  @param p_jugador_id Identificador nic per cada jugador.
  *  @return PKG_UTILS.C_OK: Si tot s'ha executat correctament <BR>
  *          PKG_UTILS.C_NOT_FOUND: Indicar que no hem pogut trobar el jugador a esborrar. <BR>
  *          PKG_UTILS.C_UNKNOWN_KO: Error desconegut, veure registre en la taula AUDITORIA. <BR>
  *  @author Juanjo Daz Prez
  *  @version 1.0 Mar-Juny 2013
  **/
  FUNCTION FN_BAIXA_JUGADOR( p_jugador_id IBDB.JUGADOR.ID%TYPE) RETURN INTEGER;

  /** Funci que actualitza la informaci d'un jugador.
  *
  *  Funci auditada per PKG_UTILS.PR_AUDITAR_CANVIS
  *  S'ha de tenir en compte que els segents triggers es veuen utilitzats durant la crida d'aquesta funci:
  *    TRG_AUDIT_JUGADOR <BR>
  *  @param p_jugador Cont la informaci actualitzada del jugador.
  *  @return PKG_UTILS.C_OK: Si tot s'ha executat correctament <BR>
  *          PKG_UTILS.C_UNKNOWN_KO: Error desconegut, veure registre en la taula AUDITORIA. <BR>
  *  @author Juanjo Daz Prez
  *  @version 1.0 Mar-Juny 2013
  **/
  FUNCTION FN_MODIFICACIO_JUGADOR( p_jugador IBDB.JUGADOR%ROWTYPE) RETURN INTEGER;

  /** Funci que dona de alta un contracte assegurant la seva integritat.
  *
  *   Funci auditada per PKG_UTILS.PR_AUDITAR_CANVIS<BR>
  *   .- La durada del contracte no pot ser nulla.<BR>
  *   .- El camp VALOR_ECONOMIC no admet valors, es realitza el calcul automticament<BR>
  *   S'ha de tenir en compte que els segents triggers es veuen utilitzats durant la crida d'aquesta funci:
  *    TRG_AUDIT_CONTRACTE <BR>
  *  @param p_contracte Cont la fila d'informaci especificada a la taula CONTRACTE.
  *  @return PKG_UTILS.C_OK: Si tot s'ha executat correctament <BR>
  *          PKG_UTILS.C_PK_ERROR: Error per violaci de la clau externa <BR>
  *          PKG_UTILS.C_NOT_FOUND:
  *          PKG_UTILS.C_UNKNOWN_KO: Error desconegut, veure registre en la taula AUDITORIA. <BR>
  *  @author Juanjo Daz Prez
  *  @version 1.0 Mar-Juny 2013
  **/
  FUNCTION FN_ALTA_CONTRACTES( p_contracte IBDB.CONTRACTE%ROWTYPE) RETURN INTEGER;

  /** Funci que dona de baixa un contracte.
  *
  *  Funci auditada per PKG_UTILS.PR_AUDITAR_CANVIS
  *  S'ha de tenir en compte que els segents triggers es veuen utilitzats durant la crida d'aquesta funci:
  *    TRG_AUDIT_CONTRACTE<BR>
  *  @param p_contracte_id Identificador nic de contracte.
  *  @return PKG_UTILS.C_OK: Si tot s'ha executat correctament <BR>
  *          PKG_UTILS.C_UNKNOWN_KO: Error desconegut, veure registre en la taula AUDITORIA. <BR>
  *  @author Juanjo Daz Prez
  *  @version 1.0 Mar-Juny 2013
  **/
  FUNCTION FN_BAIXA_CONTRACTES( p_contracte_id IBDB.CONTRACTE.ID%TYPE) RETURN INTEGER;

  /** Funci que actualitza les dades de un contracte assegurant la seva integritat.
  *
  *   Funci auditada per PKG_UTILS.PR_AUDITAR_CANVIS.
  *
  *   .- La durada del contracte no pot ser nulla.
  *   .- El camp VALOR_ECONOMIC es sobreescriu i es realitza el calcul automticament
  *  @param p_contracte Cont la fila d'informaci especificada a la taula CONTRACTE.
  *  @return PKG_UTILS.C_OK: Si tot s'ha executat correctament <BR>
  *          PKG_UTILS.C_UNKNOWN_KO: Error desconegut, veure registre en la taula AUDITORIA. <BR>
  *  @author Juanjo Daz Prez
  *  @version 1.0 Mar-Juny 2013
  **/
  FUNCTION FN_MODIFICACIO_CONTRACTES( p_contracte IBDB.CONTRACTE%ROWTYPE) RETURN INTEGER;


  /** Funci que dona d'alta un partit assegurant la seva integritat.
  *
  *   Funci auditada per PKG_UTILS.PR_AUDITAR_CANVIS
  *   S'ha de tenir en compte que els segents triggers es veuen utilitzats durant la crida d'aquesta funci:
  *  @param p_partit Cont la fila d'informaci especificada a la taula PARTIT.
  *  @return PKG_UTILS.C_OK: Si tot s'ha executat correctament <BR>
  *          PKG_UTILS.C_NOT_FOUND: Un dels dos equips no existeix, per mes informaci veure registre en la taula AUDITORIA <BR>
  *          PKG_UTILS.C_UNKNOWN_KO: Error desconegut, veure registre en la taula AUDITORIA. <BR>
  *  @author Juanjo Daz Prez
  *  @version 1.0 Mar-Juny 2013
  **/
  FUNCTION FN_ALTA_PARTITS( p_partit IBDB.PARTIT%ROWTYPE) RETURN INTEGER;

  /** Funci que dona de baixa un partit assegurant la seva integritat.
  *
  *   Funci auditada per PKG_UTILS.PR_AUDITAR_CANVIS
  *   S'ha de tenir en compte que els segents triggers es veuen utilitzats durant la crida d'aquesta funci:
  *    TRG_AUDIT_PARTIT <BR>
  *  @param p_partit_id Identificador nic de la taula de PARTIT.
  *  @return PKG_UTILS.C_OK: Si tot s'ha executat correctament <BR>
  *          PKG_UTILS.C_UNKNOWN_KO: Error desconegut, veure registre en la taula AUDITORIA. <BR>
  *  @author Juanjo Daz Prez
  *  @version 1.0 Mar-Juny 2013
  **/
  FUNCTION FN_BAIXA_PARTITS( p_partit_id IBDB.PARTIT.ID%TYPE) RETURN INTEGER;

  /** Funci que permet modificar les dades d'un partit assegurant la seva integritat.
  *
  *   Funci auditada per PKG_UTILS.PR_AUDITAR_CANVIS
  *  @param p_partit Cont la fila d'informaci especificada a la taula PARTIT.
  *  @return PKG_UTILS.C_OK: Si tot s'ha executat correctament <BR>
  *          PKG_UTILS.C_UNKNOWN_KO: Error desconegut, veure registre en la taula AUDITORIA. <BR>
  *  @author Juanjo Daz Prez
  *  @version 1.0 Mar-Juny 2013
  **/
  FUNCTION FN_MODIFICACIO_PARTITS( p_partit IBDB.PARTIT%ROWTYPE) RETURN INTEGER;

  /** Funci que permtet donar d'alta un arbitre assegurant la seva integritat.
  *
  *   Funci auditada per PKG_UTILS.PR_AUDITAR_CANVIS
  *     IBDB.TRG_NOU_ARBITRE <BR>
  *  @param p_partit Cont la fila d'informaci especificada a la taula ARBITRE_PARTIT.
  *  @return PKG_UTILS.C_OK: Si tot s'ha executat correctament <BR>
  *          PKG_UTILS.C_PK_ERROR: Aquest retorn es pot donar o be perque s'ha exedit del nmero de arbitres permesos o be per intentar insertar el mateix registre dos cops.<BR>
  *          PKG_UTILS.C_NOT_FOUND: L'arbitre o el partit que es volen relacionar no existeix .<BR>
  *          PKG_UTILS.C_UNKNOWN_KO: Error desconegut, veure registre en la taula AUDITORIA. <BR>
  *  @author Juanjo Daz Prez
  *  @version 1.0 Mar-Juny 2013
  **/
  FUNCTION FN_ALTA_ARBITRE_PARTIT (pArbitrePartit IBDB.ARBITRE_PARTIT%ROWTYPE) RETURN INTEGER;

  /** Funci que permtet modificar un arbitre assegurant la seva integritat.
  *
  *   Funci auditada per PKG_UTILS.PR_AUDITAR_CANVIS
  *
  *  @param pArbitrePartit Cont la fila d'informaci especificada a la taula ARBITRE_PARTIT.
  *  @return PKG_UTILS.C_OK: Si tot s'ha executat correctament <BR>
  *          PKG_UTILS.C_UNKNOWN_KO: Error desconegut, veure registre en la taula AUDITORIA. <BR>
  *  @author Juanjo Daz Prez
  *  @version 1.0 Mar-Juny 2013
  **/
  FUNCTION FN_MODIFICACIO_ARBITRE_PARTIT(pArbitrePartit IBDB.ARBITRE_PARTIT%ROWTYPE) RETURN INTEGER;

  /** Funci que permtet donar de baixa un arbitre assegurant la seva integritat.
  *
  *   Funci auditada per PKG_UTILS.PR_AUDITAR_CANVIS
  *
  *  @param pArbitrePartit Cont la fila d'informaci especificada a la taula ARBITRE_PARTIT.
  *  @return PKG_UTILS.C_OK: Si tot s'ha executat correctament <BR>
  *          PKG_UTILS.C_UNKNOWN_KO: Error desconegut, veure registre en la taula AUDITORIA. <BR>
  *  @author Juanjo Daz Prez
  *  @version 1.0 Mar-Juny 2013
  **/
  FUNCTION FN_BAIXA_ARBITRE_PARTIT( p_arbitre_id IBDB.ARBITRE_PARTIT.ID_ARBITRE%TYPE, p_partit_id IBDB.ARBITRE_PARTIT.ID_PARTIT%TYPE ) RETURN INTEGER;

  /** Funci que permtet donar d'alta les estadstiques d'un partit assegurant la seva integritat.
  *
  *   La funci te cura de no permetre valors negatius en aquells camps que no son calculat i que per tant, l'usuari te la possiblitat d'omplenar. <BR>
  *   Tanmateix aquesta funci no s'encarrega de validar que el valor de els tirs llirues encistellats sigui menor que el dels tirs lliures intentants. <BR>
  *   El camp PUNTS es un camp calculat i que no encara que l'usuari el vulgui omplenar, ser sobreescrit pel segent clcul.<BR>
  *       PUNTS := NVL(TIRS_LLIURES_ENCISTELLATS,0) + (NVL(DOS_PUNTS_ENCISTELLATS,0)*2) + (NVL(TRES_PUNTS_ENCISTELLATS,0)*3)<BR>
  *   Funci auditada per PKG_UTILS.PR_AUDITAR_CANVIS <BR>
  *     <BR>
  *  La funci crida al procediment definit en el body d'aquest paquet anomenat PR_ESTADISTIQUES_EST_PARTIT <BR>
  *  @return PKG_UTILS.C_OK: Si tot s'ha executat correctament <BR>
  *          PKG_UTILS.C_UNKNOWN_KO: Error desconegut, veure registre en la taula AUDITORIA. <BR>
  *
  *  @author Juanjo Daz Prez
  *  @version 1.0 Mar-Juny 2013
  **/
  FUNCTION FN_ALTA_ESTAD_PARTIT_JUGADOR(pEstadisticaPartit IBDB.ESTADISTICA_PARTIT%ROWTYPE) RETURN INTEGER;

  /** Funci que permtet modificar les estadstiques d'un partit assegurant la seva integritat.
  *
  *   Funci auditada per PKG_UTILS.PR_AUDITAR_CANVIS
  *
  *  La funci crida al procediment definit en el body d'aquest paquet anomenat PR_ESTADISTIQUES_EST_PARTIT <BR>
  *  La funci fa una crida interna a FN_VALORACIO_JUGADOR i FN_VALORACIO_JUGADOR_PONDERADA. <BR>
  *  S'ha de pensar que l'estadstica partit quedar tal com s'especifiqui en el parmetre pEstadisticaPartit. Es a dir
  *  l'usuari ha d'omplenar el registre tal y com vol que quedi.
  *
  *  @param pEstadisticaPartit Cont la fila d'informaci especificada a la taula ESTADISTICA_PARTIT.<BR>
  *  @return PKG_UTILS.C_OK: Si tot s'ha executat correctament <BR>
  *          PKG_UTILS.C_UNKNOWN_KO: Error desconegut, veure registre en la taula AUDITORIA. <BR>
  *  @author Juanjo Daz Prez
  *  @version 1.0 Mar-Juny 2013
  **/
  FUNCTION FN_MODIFICA_EST_PARTIT_JUGADOR(pEstadisticaPartit IBDB.ESTADISTICA_PARTIT%ROWTYPE) RETURN INTEGER;

  /** Funci que permtet esborrar estadstiques d'un partit assegurant la seva integritat.<BR>
  *
  *   Encara que el parmetre de la funci requereix crear un registre complert de la taula ESTADISTICA_PARTIT pel bon funcionament
  *   de la funci noms ser necesari omplenar ID_JUGADOR i ID_PARTIT ja que aquest sn la clau primaria de la taula ESTADISTICA_PARTIT .<BR>
  *   Funci auditada per PKG_UTILS.PR_AUDITAR_CANVIS
  *
  *  La funci crida al procediment definit en el body d'aquest paquet anomenat PR_ESTADISTIQUES_EST_PARTIT <BR>
  *  @return PKG_UTILS.C_OK: Si tot s'ha executat correctament <BR>
  *          PKG_UTILS.C_UNKNOWN_KO: Error desconegut, veure registre en la taula AUDITORIA. <BR>
  *  @author Juanjo Daz Prez
  *  @version 1.0 Mar-Juny 2013
  **/
  FUNCTION FN_BORRA_EST_PARTIT_JUGADOR(pEstadisticaPartit IBDB.ESTADISTICA_PARTIT%ROWTYPE) RETURN INTEGER;

END PKG_GESTIO;
/
CREATE OR REPLACE PACKAGE BODY IBDB.PKG_GESTIO AS

  /** Paquet que conte les funcions de gesti.
  *
  *  @author Juanjo Daz Prez
  *  @version 1.0 Mar-Juny 2013
  **/

  /* Error definit per capturar un error de clau externa.*/
  FK_EXCEPTION  EXCEPTION;

  /* Error definit per validar la regla de negocia referent al nmero de federat duplicat dins del mateix pas.*/
  E_NUM_FEDERAT_DUPL  EXCEPTION;

  /* Error definit per validar si el subestat d'un jugador es correcte*/
  E_SUB_ESTAT_INCORRECTE EXCEPTION;

  /* Nmero assignat al error FK_EXCEPTION */
  PRAGMA EXCEPTION_INIT (FK_EXCEPTION, -20001);
  /* Nmero assignat al error E_NUM_FEDERAT_DUPL */
  PRAGMA EXCEPTION_INIT( E_NUM_FEDERAT_DUPL, -20002 );
  /* Nmero assignat al error E_SUB_ESTAT_INCORRECTE */
  PRAGMA EXCEPTION_INIT( E_SUB_ESTAT_INCORRECTE, -20003 );

  /** Procediment que agrupa les accions que s'han de realitzar al aplicar alguna accio sobre els jugadors
  *
  *  @author Juanjo Daz Prez
  *  @version 1.0 Mar-Juny 2013
  **/
  PROCEDURE PR_ESTADISTIQUES_JUGADOR IS
  BEGIN
    -- Actualitzem estadistiques.
    IBDWH.pkg_dwh_estadistiques.PR_JUGADORS_EN_ACTIU;
  END PR_ESTADISTIQUES_JUGADOR;


  /** Procediment que agrupa les accions que s'han de realitzar al aplicar alguna accio sobre els contractes
  *
  *  @author Juanjo Daz Prez
  *  @version 1.0 Mar-Juny 2013
  **/
  PROCEDURE PR_ESTADISTIQUES_CONTRACTE IS
  BEGIN
    -- Actualitzem estadistiques.
    IBDWH.pkg_dwh_estadistiques.PR_JUGADORS_AMB_MES_GUANYS;
    IBDWH.pkg_dwh_estadistiques.PR_JUGADOR_MILLOR_PAGAT;
    IBDWH.pkg_dwh_estadistiques.PR_JUGADORS_ACABEN_CONTRACTE;
    IBDWH.pkg_dwh_estadistiques.PR_NUM_CONT_VALOR_ECONOMIC;
    IBDWH.pkg_dwh_estadistiques.PR_SOU_MIG_ANUAL_JUGADOR;
    IBDWH.pkg_dwh_estadistiques.PR_TOP_10_EQUIPS_COMPRADORS;
  END;


  /** Procediment que agrupa les accions que s'han de realitzar al aplicar alguna accio sobre la taula ESTADISTICA_PARTIT
  *
  *  @author Juanjo Daz Prez
  *  @version 1.0 Mar-Juny 2013
  **/
  PROCEDURE PR_ESTADISTIQUES_EST_PARTIT(pJugadorID IN IBDB.JUGADOR.ID%TYPE,
                                        pPartitID IN IBDB.PARTIT.ID%TYPE) IS
  BEGIN


    FOR vRegistre in (SELECT DISTINCT P.COMPETICIO_ID, C.TEMPORADA, EP.ID_JUGADOR
                        FROM IBDB.ESTADISTICA_PARTIT EP,
                             IBDB.PARTIT P,
                             IBDB.COMPETICIO C
                       WHERE EP.ID_PARTIT = P.ID
                         AND C.ID = P.COMPETICIO_ID
                         AND EP.ID_JUGADOR = NVL(pJugadorID,EP.ID_JUGADOR)
                         AND EP.ID_PARTIT = NVL(pPartitID, P.ID)) LOOP

      IBDWH.pkg_dwh_estadistiques.PR_JUGADORS_EN_UNA_COMPETICIO(pJugadorID => vRegistre.Id_Jugador);
      IBDWH.pkg_dwh_estadistiques.PR_MAXIM_ANOTADOR_COMPETICIO(pCompeticioID => vRegistre.Competicio_Id);
      IBDWH.pkg_dwh_estadistiques.PR_MILLORS_EQUIPS(pCompeticioID => vRegistre.Competicio_Id,
                                                    pTemporada    => vRegistre.Temporada);
      IBDWH.pkg_dwh_estadistiques.PR_MILLOR_JUGADOR_DEL_MON(pCompeticioID => vRegistre.Competicio_Id);
    END LOOP;

    IBDWH.pkg_dwh_estadistiques.PR_EQUIPS_COMP_PUNTS_FAVOR;
    IBDWH.pkg_dwh_estadistiques.PR_TOP_5_JUGADORS_POSICIO;

  END PR_ESTADISTIQUES_EST_PARTIT;

  /* Funci que dona de alta un contracte assegurant la seva integritat. */
  FUNCTION FN_ALTA_JUGADOR( p_jugador IBDB.JUGADOR%ROWTYPE) RETURN INTEGER IS
    vNomObjecte VARCHAR(31) := 'FN_ALTA_JUGADOR';
    vJugador IBDB.JUGADOR%ROWTYPE := p_jugador;
    vNumFederatDuplicat INTEGER(1);

  BEGIN

   /*
   Numero que proveir la federaci, aquest nmero no te perqu se nic
   entre jugadors de diferents pasos, per si que ho haur de ser entre jugadors del mateix pas.
   */
   BEGIN
     SELECT 1
       INTO vNumFederatDuplicat
       FROM IBDB.JUGADOR J
      WHERE J.NUMERO_FEDERAT = p_jugador.numero_federat
        AND J.NACIONALITAT_ID = p_jugador.nacionalitat_id;

     RAISE E_NUM_FEDERAT_DUPL;
   EXCEPTION
      WHEN NO_DATA_FOUND THEN
        NULL; -- CORRECTE!
      WHEN E_NUM_FEDERAT_DUPL THEN
        RAISE_APPLICATION_ERROR( -20002, 'Numero de federat per pais duplicat (Num. Federat, Nacionalitat) : ' || p_jugador.numero_federat || ',' || p_jugador.nacionalitat_id );
   END;

   /*
   Noms es permetr guardar si el jugador est de baixa mdica o baixa per motius personals en cas
   de que el seu estat sigui de baixa.
   */
   IF (p_jugador.ESTAT = 'ALTA' AND p_jugador.SUB_ESTAT IS NOT NULL) THEN
     RAISE_APPLICATION_ERROR( -20002, 'Estat = ALTA i SUBESTAT informat');
   END IF;

    -- No es permet que ni la alada ni el pes del jugador tinguin valors negatius.
    vJugador.Alcada := abs(vJugador.Alcada);
    vJugador.Pes := abs(vJugador.Pes);

    -- Valida que el id no estigui vuit, i si ho est en crea un de nou
    INSERT INTO IBDB.JUGADOR VALUES vJugador;
    -- Actualitzem estadistiques.
    PR_ESTADISTIQUES_JUGADOR;
    COMMIT;

    -- Auditem les accions realitzades
    IBDB.PKG_UTILS.PR_AUDITAR_CANVIS(vNomObjecte, 'INSERT OK - JUGADOR_ID:' || vJugador.id || ' -  ' || SQLERRM );

    RETURN IBDB.PKG_UTILS.C_OK;

  EXCEPTION
    WHEN DUP_VAL_ON_INDEX THEN
      IBDB.PKG_UTILS.PR_AUDITAR_CANVIS(vNomObjecte, 'DUP_VAL_ON_INDEX - JUGADOR_ID:' || vJugador.id || ' -  ' || SQLERRM );
      RETURN IBDB.PKG_UTILS.C_PK_ERROR;
    WHEN FK_EXCEPTION THEN
      IBDB.PKG_UTILS.PR_AUDITAR_CANVIS(vNomObjecte, 'FK_EXCEPTION - ( NACIONALITAT_ID | FEDERACIO_ID | POSICIO_ID | REPRESENTANT_ID )  - ' ||
                                                                   '(' || vJugador.NACIONALITAT_ID || '|'
                                                                       || vJugador.FEDERACIO_ID    || '|'
                                                                       || vJugador.POSICIO_ID      || '|'
                                                                       || vJugador.REPRESENTANT_ID || ')'
                                                                       || ' # ' || SQLERRM);
      RETURN IBDB.PKG_UTILS.C_FK_ERROR;
    WHEN OTHERS THEN
      IBDB.PKG_UTILS.PR_AUDITAR_CANVIS(vNomObjecte, 'OTHERS - JUGADOR_ID:' || vJugador.id || ' -  ' || SQLERRM );
      RETURN IBDB.PKG_UTILS.C_UNKNOWN_KO;
  END FN_ALTA_JUGADOR;

  /* Funci que dona de baix un jugador. */
  FUNCTION FN_BAIXA_JUGADOR( p_jugador_id IBDB.JUGADOR.ID%TYPE) RETURN INTEGER IS
    vNomObjecte VARCHAR(31) := 'FN_BAIXA_JUGADOR';
  BEGIN



     DELETE FROM IBDB.JUGADOR J WHERE J.ID = p_jugador_id;

     -- Si no hem pogut esborrar el jugador llanarem una excepci per avisar al usuari.
     IF (sql%rowcount = 0) THEN
       ROLLBACK;
       RAISE NO_DATA_FOUND;
     END IF;
     /*Al donar de baixa un jugador existent (eliminar-lo) la informaci relacionada a
       aquest jugador tamb seliminara.
          CONTRACTE
          ESTADISTICA_PARTIT
          DIAGNOSTIC
     */

     DELETE FROM IBDB.ESTADISTICA_PARTIT EP WHERE EP.ID_JUGADOR = p_jugador_id;
     IBDB.PKG_UTILS.PR_AUDITAR_CANVIS(vNomObjecte, 'Jugador esborrat de ESTADISTICA_PARTIT ' ||  p_jugador_id);

     DELETE FROM IBDB.DIAGNOSTIC D WHERE D.JUGADOR_ID = p_jugador_id;
     IBDB.PKG_UTILS.PR_AUDITAR_CANVIS(vNomObjecte, 'Jugador esborrat de DIAGNOSTIC ' ||  p_jugador_id);

     DELETE FROM IBDB.CONTRACTE C WHERE C.JUGADOR_ID = p_jugador_id;
     IBDB.PKG_UTILS.PR_AUDITAR_CANVIS(vNomObjecte, 'Jugador esborrat de CONTRACTE ' ||  p_jugador_id);

     -- Actualitzem estadistiques.
     PR_ESTADISTIQUES_JUGADOR;
     PR_ESTADISTIQUES_EST_PARTIT(p_jugador_id, null);
     PR_ESTADISTIQUES_CONTRACTE;
     COMMIT;

     -- Auditem les accions realitzades
     IBDB.PKG_UTILS.PR_AUDITAR_CANVIS(vNomObjecte, 'ID_JUGADOR: ' || p_jugador_id );

     RETURN IBDB.PKG_UTILS.C_OK;

  EXCEPTION
    WHEN NO_DATA_FOUND THEN
      IBDB.PKG_UTILS.PR_AUDITAR_CANVIS(vNomObjecte, 'NO_DATA_FOUND -ID_JUGADOR:' || p_jugador_id);
      RETURN IBDB.PKG_UTILS.C_NOT_FOUND;
    WHEN OTHERS THEN
      IBDB.PKG_UTILS.PR_AUDITAR_CANVIS(vNomObjecte, 'OTHERS - ERROR:' || SQLERRM);
      RETURN IBDB.PKG_UTILS.C_UNKNOWN_KO;
  END FN_BAIXA_JUGADOR ;

  /* Funci que actualitza la informaci d'un jugador. */
  FUNCTION FN_MODIFICACIO_JUGADOR( p_jugador IBDB.JUGADOR%ROWTYPE) RETURN INTEGER IS
    vNomObjecte VARCHAR(31) := 'FN_MODIFICACIO_JUGADOR';
    vJugador IBDB.JUGADOR%ROWTYPE := p_jugador;
  BEGIN

    /* Actualitzem la row sencera amb la informaci que ens han donat a travs del parametre p_jugador
       Si aquesta nova informaci no seguieix les regles que hem posat a la taula, la funci sortir
       per l'excepci i registrarem l'error en la taula d'auditoria.
    */
    UPDATE IBDB.JUGADOR J
      SET ROW = vJugador
    WHERE J.ID = vJugador.id;
    -- Actualitzem estadistiques.
    PR_ESTADISTIQUES_JUGADOR;
    COMMIT;

    -- Auditem les accions realitzades
    IBDB.PKG_UTILS.PR_AUDITAR_CANVIS(vNomObjecte, 'ID_JUGADOR: ' || vJugador.id );

    RETURN IBDB.PKG_UTILS.C_OK;
  EXCEPTION
    WHEN OTHERS THEN
      IBDB.PKG_UTILS.PR_AUDITAR_CANVIS(vNomObjecte, 'OTHERS - ERROR:' || SQLERRM);
      RETURN IBDB.PKG_UTILS.C_UNKNOWN_KO;
  END FN_MODIFICACIO_JUGADOR;

  /* Funci que dona de alta un contracte assegurant la seva integritat. */
  FUNCTION FN_ALTA_CONTRACTES( p_contracte IBDB.CONTRACTE%ROWTYPE) RETURN INTEGER IS
    vNomObjecte VARCHAR(31) := 'FN_ALTA_CONTRACTES';
    vContracte IBDB.CONTRACTE%ROWTYPE := p_contracte;
  BEGIN

    -- La durada del contracte no pot ser negativa
    vContracte.Durada_Contracte := ABS(vContracte.Durada_Contracte);
    -- Valor econmic es el valor de la compensaci mes el salari brut anual del jugador.
    vContracte.VALOR_ECONOMIC := NVL(vContracte.COMPENSACIO,0) + NVL(vContracte.SALARI_BRUT_ANUAL,0);

    INSERT INTO IBDB.CONTRACTE VALUES vContracte;
    -- Actualitzem estadistiques.
    PR_ESTADISTIQUES_CONTRACTE;
    COMMIT;

    RETURN IBDB.PKG_UTILS.C_OK;
  EXCEPTION
    WHEN DUP_VAL_ON_INDEX THEN
      IBDB.PKG_UTILS.PR_AUDITAR_CANVIS(vNomObjecte, 'DUP_VAL_ON_INDEX - CONTRACTE_ID:' || vContracte.ID || ' -  ' || SQLERRM );
      RETURN IBDB.PKG_UTILS.C_PK_ERROR;
    WHEN FK_EXCEPTION THEN
      IBDB.PKG_UTILS.PR_AUDITAR_CANVIS(vNomObjecte, 'FK_EXCEPTION - ( JUGADOR_ID | REPRESENTANT_ID | EQUIP_COMPRADOR_ID | EQUIP_VENEDOR_ID  )  - ' ||
                                                                   '(' || vContracte.JUGADOR_ID || '|'
                                                                       || vContracte.REPRESENTANT_ID  || '|'
                                                                       || vContracte.EQUIP_COMPRADOR_ID  || '|'
                                                                       || vContracte.EQUIP_VENEDOR_ID || '|)'
                                                                       || ' # ' || SQLERRM);
      RETURN IBDB.PKG_UTILS.C_NOT_FOUND;
    WHEN OTHERS THEN
       IBDB.PKG_UTILS.PR_AUDITAR_CANVIS(vNomObjecte, 'OTHERS - ERROR:' || SQLERRM);
       RETURN PKG_UTILS.C_UNKNOWN_KO;
  END FN_ALTA_CONTRACTES;

  /* Funci que dona de baixa un contracte. */
  FUNCTION FN_BAIXA_CONTRACTES( p_contracte_id IBDB.CONTRACTE.ID%TYPE) RETURN INTEGER IS
    vNomObjecte VARCHAR(31) := 'FN_BAIXA_CONTRACTES';
  BEGIN

    -- En cas de no existir el contracte donar l'excepci de NOT_FOUND.
    DELETE FROM IBDB.CONTRACTE C WHERE C.ID = p_contracte_id;
    -- Actualitzem estadistiques.
    PR_ESTADISTIQUES_CONTRACTE;
    COMMIT;

    IBDB.PKG_UTILS.PR_AUDITAR_CANVIS(vNomObjecte, 'CONTRACTE_ID: ' || p_contracte_id  );
    RETURN IBDB.PKG_UTILS.C_OK;
  EXCEPTION
    WHEN NO_DATA_FOUND THEN
       IBDB.PKG_UTILS.PR_AUDITAR_CANVIS(vNomObjecte, 'NO DATA FOUND - ERROR CONTRACTE_ID: ' || p_contracte_id  );
       RETURN IBDB.PKG_UTILS.C_NOT_FOUND;
    WHEN OTHERS THEN
       IBDB.PKG_UTILS.PR_AUDITAR_CANVIS(vNomObjecte,  'OTHERS - ERROR:' || SQLERRM);
       RETURN PKG_UTILS.C_UNKNOWN_KO;
  END FN_BAIXA_CONTRACTES;

  /* Funci que actualitza les dades de un contracte assegurant la seva integritat. */
  FUNCTION FN_MODIFICACIO_CONTRACTES( p_contracte IBDB.CONTRACTE%ROWTYPE) RETURN INTEGER IS
    vNomObjecte VARCHAR(31) := 'FN_MODIFICACIO_CONTRACTES';
    vContracte IBDB.CONTRACTE%ROWTYPE := p_contracte;
  BEGIN

    IBDB.PKG_UTILS.PR_AUDITAR_CANVIS(vNomObjecte, 'CONTRACTE_ID: ' || vContracte.ID );

    UPDATE IBDB.CONTRACTE C
       SET ROW = vContracte
     WHERE C.ID = vContracte.id;
    -- Actualitzem estadistiques.
    PR_ESTADISTIQUES_CONTRACTE;
    COMMIT;

    RETURN IBDB.PKG_UTILS.C_OK;

  EXCEPTION
    WHEN OTHERS THEN
      IBDB.PKG_UTILS.PR_AUDITAR_CANVIS(vNomObjecte, 'OTHERS - ERROR:' || SQLERRM);
      RETURN IBDB.PKG_UTILS.C_UNKNOWN_KO;
  END FN_MODIFICACIO_CONTRACTES;

  /* Funci que dona d'alta un partit assegurant la seva integritat. */
  FUNCTION FN_ALTA_PARTITS( p_partit IBDB.PARTIT%ROWTYPE) RETURN INTEGER IS
    vNomObjecte VARCHAR(31) := 'FN_ALTA_PARTITS';
    vPartit IBDB.PARTIT%ROWTYPE := p_partit;
    vExisteixen NUMBER(1);
  BEGIN

    -- Es valida que els equips que disputaran el partit existeixen.
    SELECT count(1)
      INTO vExisteixen
      FROM IBDB.EQUIP E
     WHERE E.ID IN (vPartit.Equip_Local_Id, vPartit.Equip_Visitant_Id);

    IF (vExisteixen < 2 ) THEN
      RAISE FK_EXCEPTION;
    END IF;

    -- A partir de la clau primaria i claus externes es controla la integritat d'aquesta taula.
    INSERT INTO IBDB.PARTIT VALUES vPartit;
    COMMIT;

    RETURN IBDB.PKG_UTILS.C_OK;

  EXCEPTION
    WHEN DUP_VAL_ON_INDEX THEN
      IBDB.PKG_UTILS.PR_AUDITAR_CANVIS(vNomObjecte, 'DUP_VAL_ON_INDEX - PARTIT_ID:' || vPartit.id || ' -  ' || SQLERRM );
      RETURN IBDB.PKG_UTILS.C_PK_ERROR;
    WHEN FK_EXCEPTION THEN
      IBDB.PKG_UTILS.PR_AUDITAR_CANVIS(vNomObjecte, 'FK_EXCEPTION - ( EQUIP_LOCAL_ID | EQUIP_VISITANT_ID | COMPETICIO_ID)  - ' ||
                                                                   '(' || vPartit.EQUIP_LOCAL_ID     || '|'
                                                                       || vPartit.EQUIP_VISITANT_ID || '|'
                                                                       || vPartit.COMPETICIO_ID     || ')'
                                                                       || ' # ' || SQLERRM);
      RETURN IBDB.PKG_UTILS.C_NOT_FOUND;
    WHEN OTHERS THEN
      IBDB.PKG_UTILS.PR_AUDITAR_CANVIS(vNomObjecte, 'OTHERS - ERROR:' || SQLERRM );
      RETURN IBDB.PKG_UTILS.C_UNKNOWN_KO;
  END FN_ALTA_PARTITS;

  /* Funci que dona de baixa un partit assegurant la seva integritat. */
  FUNCTION FN_BAIXA_PARTITS( p_partit_id IBDB.PARTIT.ID%TYPE) RETURN INTEGER IS
    vNomObjecte VARCHAR(31) := 'FN_BAIXA_PARTITS';
  BEGIN

    DELETE FROM IBDB.PARTIT P WHERE P.ID = p_partit_id;
    -- Actualitzem estadistiques
    PR_ESTADISTIQUES_EST_PARTIT(pJugadorID => NULL,
                                pPartitID  => p_partit_id);
    COMMIT;
    IBDB.PKG_UTILS.PR_AUDITAR_CANVIS(vNomObjecte, 'PARTIT_ID: ' || p_partit_id);
    RETURN IBDB.PKG_UTILS.C_OK;
  EXCEPTION
    WHEN OTHERS THEN
      IBDB.PKG_UTILS.PR_AUDITAR_CANVIS(vNomObjecte, 'OTHERS - ERROR:' || SQLERRM);
      RETURN IBDB.PKG_UTILS.C_UNKNOWN_KO;
  END FN_BAIXA_PARTITS;

  /* Funci que permet modificar les dades d'un partit assegurant la seva integritat. */
  FUNCTION FN_MODIFICACIO_PARTITS( p_partit IBDB.PARTIT%ROWTYPE) RETURN INTEGER IS
    vNomObjecte VARCHAR(31) := 'FN_MODIFICACIO_PARTITS';
    vPartit IBDB.PARTIT%ROWTYPE := p_partit;
  BEGIN

    IBDB.PKG_UTILS.PR_AUDITAR_CANVIS(vNomObjecte, 'PARTIT_ID: ' || vPartit.ID);

    UPDATE IBDB.PARTIT P
       SET ROW = vPartit
     WHERE P.ID = vPartit.id;
    -- Actualitzem estadistiques
    PR_ESTADISTIQUES_EST_PARTIT(pJugadorID => NULL,
                                pPartitID  => vPartit.ID);
    COMMIT;
    RETURN IBDB.PKG_UTILS.C_OK;
  EXCEPTION
    WHEN OTHERS THEN
      IBDB.PKG_UTILS.PR_AUDITAR_CANVIS(vNomObjecte, 'OTHERS - ERROR:' || SQLERRM);
      RETURN IBDB.PKG_UTILS.C_UNKNOWN_KO;
  END FN_MODIFICACIO_PARTITS;

  /* Funci que permtet donar d'alta un arbitre assegurant la seva integritat. */
  FUNCTION FN_ALTA_ARBITRE_PARTIT(pArbitrePartit IBDB.ARBITRE_PARTIT%ROWTYPE) RETURN INTEGER IS
    vNomObjecte VARCHAR2(31) := 'FN_ALTA_ARBITRE_PARTIT';
    vCanInsert INTEGER := 0;

    E_MAX_NUM_ARB_X_PARTIT  EXCEPTION;
  BEGIN

      /* Controlem que l'arbitre existeix */
      BEGIN
        SELECT CASE
                  WHEN (MAX(c.nombre_arbitres) <= COUNT(AP.ID_ARBITRE)) THEN
                    0
                  ELSE
                    1
                END CAN_INSERT
           INTO vCanInsert
           FROM IBDB.COMPETICIO C,
                IBDB.PARTIT P,
                IBDB.ARBITRE_PARTIT AP
          WHERE P.COMPETICIO_ID = C.ID
            AND AP.ID_PARTIT(+) = P.ID
            AND P.ID = pArbitrePartit.ID_PARTIT;
       EXCEPTION
         WHEN NO_DATA_FOUND THEN
           RAISE FK_EXCEPTION;
       END;

       /* Controlem que el nmero d'arbitres estigui dins dels parametres permesos   */
       IF (vCanInsert = 0) THEN
         RAISE E_MAX_NUM_ARB_X_PARTIT;
       END IF;

       INSERT INTO IBDB.ARBITRE_PARTIT
       VALUES pArbitrePartit;
       COMMIT;
       RETURN IBDB.PKG_UTILS.C_OK;
  EXCEPTION
    WHEN E_MAX_NUM_ARB_X_PARTIT THEN
      IBDB.PKG_UTILS.PR_AUDITAR_CANVIS(vNomObjecte, 'E_MAX_NUM_ARB_X_PARTIT - PARTIT_ID:' || pArbitrePartit.ID_PARTIT || ' -  ' || SQLERRM );
      RETURN IBDB.PKG_UTILS.C_PK_ERROR;
    WHEN DUP_VAL_ON_INDEX THEN
      IBDB.PKG_UTILS.PR_AUDITAR_CANVIS(vNomObjecte, 'DUP_VAL_ON_INDEX - PARTIT_ID:' || pArbitrePartit.ID_PARTIT || ' -  ' || SQLERRM );
      RETURN IBDB.PKG_UTILS.C_PK_ERROR;
    WHEN FK_EXCEPTION THEN
      IBDB.PKG_UTILS.PR_AUDITAR_CANVIS(vNomObjecte, 'FK_EXCEPTION - ( ID_ARBITRE | ID_PARTIT )  - ' ||
                                                                   '(' || pArbitrePartit.ID_ARBITRE     || '|'
                                                                       || pArbitrePartit.ID_PARTIT ||  ')'
                                                                       || ' # ' || SQLERRM);
      RETURN IBDB.PKG_UTILS.C_NOT_FOUND;
    WHEN OTHERS THEN
      IBDB.PKG_UTILS.PR_AUDITAR_CANVIS(vNomObjecte, 'OTHERS - ERROR:' || SQLERRM );
      RETURN IBDB.PKG_UTILS.C_UNKNOWN_KO;
  END FN_ALTA_ARBITRE_PARTIT;

  /* Funci que permtet donar de baixa un arbitre assegurant la seva integritat. */
  FUNCTION FN_BAIXA_ARBITRE_PARTIT( p_arbitre_id IBDB.ARBITRE_PARTIT.ID_ARBITRE%TYPE, p_partit_id IBDB.ARBITRE_PARTIT.ID_PARTIT%TYPE ) RETURN INTEGER IS
    vNomObjecte VARCHAR(31) := 'FN_BAIXA_ARBITRE_PARTIT';
  BEGIN
    DELETE FROM IBDB.ARBITRE_PARTIT AP
     WHERE AP.ID_PARTIT = p_partit_id
       AND AP.ID_ARBITRE = p_arbitre_id;
    COMMIT;
    IBDB.PKG_UTILS.PR_AUDITAR_CANVIS(vNomObjecte, 'PARTIT_ID: ' || p_partit_id ||
                                                  'ARBITRE_ID: ' || p_arbitre_id);
    RETURN IBDB.PKG_UTILS.C_OK;
  EXCEPTION
    WHEN OTHERS THEN
      IBDB.PKG_UTILS.PR_AUDITAR_CANVIS(vNomObjecte, 'OTHERS - ERROR:' || SQLERRM);
      RETURN IBDB.PKG_UTILS.C_UNKNOWN_KO;
  END FN_BAIXA_ARBITRE_PARTIT;

  /* Funci que permtet modificar un arbitre assegurant la seva integritat. */
  FUNCTION FN_MODIFICACIO_ARBITRE_PARTIT(pArbitrePartit IBDB.ARBITRE_PARTIT%ROWTYPE) RETURN INTEGER IS
    vNomObjecte VARCHAR(31) := 'FN_MODIFICACIO_ARBITRE_PARTIT';
  BEGIN

    IBDB.PKG_UTILS.PR_AUDITAR_CANVIS(vNomObjecte, 'PARTIT_ID: ' || pArbitrePartit.ID_PARTIT ||
                                                  'ARBITRE_ID: ' || pArbitrePartit.ID_ARBITRE);

    UPDATE IBDB.ARBITRE_PARTIT AP
       SET ROW = pArbitrePartit
     WHERE AP.ID_PARTIT = pArbitrePartit.ID_PARTIT
       AND AP.ID_ARBITRE = pArbitrePartit.ID_ARBITRE;
    COMMIT;
    RETURN IBDB.PKG_UTILS.C_OK;
  EXCEPTION
    WHEN OTHERS THEN
      IBDB.PKG_UTILS.PR_AUDITAR_CANVIS(vNomObjecte, 'OTHERS - ERROR:' || SQLERRM);
      RETURN IBDB.PKG_UTILS.C_UNKNOWN_KO;
  END FN_MODIFICACIO_ARBITRE_PARTIT;


  /* Funci que relaitza el calcul de la valoraci del jugador en el partit. */
  FUNCTION FN_VALORACIO_JUGADOR(pEstadisticaPartit IBDB.ESTADISTICA_PARTIT%ROWTYPE ) RETURN INTEGER IS
    vNomObjecte VARCHAR(31) := 'FN_VALORACIO_JUGADOR';
  BEGIN
     IBDB.PKG_UTILS.PR_AUDITAR_CANVIS(vNomObjecte, pEstadisticaPartit.ID_JUGADOR || '|' || pEstadisticaPartit.ID_PARTIT  );
     /**
     Descripcio: La valoraci es calcula segons la frmula
     V = PT + T1E + T2E + T3E + RD + RO + AS + TF + PR + FR - T1I - T2I - T3I - TC - PP - FC
     **/
     RETURN NVL(pEstadisticaPartit.PUNTS,0) +
            NVL(pEstadisticaPartit.TIRS_LLIURES_ENCISTELLATS,0) +
            NVL(pEstadisticaPartit.DOS_PUNTS_ENCISTELLATS,0) +
            NVL(pEstadisticaPartit.TRES_PUNTS_ENCISTELLATS,0) +
            NVL(pEstadisticaPartit.REBOTS_DEFENSIUS,0) +
            NVL(pEstadisticaPartit.REBOTS_OFENSIUS,0) +
            NVL(pEstadisticaPartit.ASSISTENCIES,0) +
            NVL(pEstadisticaPartit.TAPS_A_FAVOR,0) +
            NVL(pEstadisticaPartit.PILOTES_RECUPERADES,0) +
            NVL(pEstadisticaPartit.FALTES_REBUDES,0) -
            NVL(pEstadisticaPartit.TIRS_LLIURES_INTENTATS,0) -
            NVL(pEstadisticaPartit.DOS_PUNTS_INTENTATS,0) -
            NVL(pEstadisticaPartit.TRES_PUNTS_INTENTATS,0) -
            NVL(pEstadisticaPartit.TAPS_EN_CONTRA,0) -
            NVL(pEstadisticaPartit.PILOTES_PERDUDES,0) -
            NVL(pEstadisticaPartit.FALTAS_COMESES,0);
  EXCEPTION
    WHEN OTHERS THEN
      IBDB.PKG_UTILS.PR_AUDITAR_CANVIS(vNomObjecte, SQLERRM);
      RETURN IBDB.PKG_UTILS.C_UNKNOWN_KO;
  END;


  /* Funci que realitza el calcul de la valoraci ponderada del jugador*/
  FUNCTION FN_VALORACIO_JUGADOR_PONDERADA(pEstadisticaPartit IBDB.ESTADISTICA_PARTIT%ROWTYPE ) RETURN INTEGER IS
    vNomObjecte VARCHAR(31) := 'FN_VALORACIO_JUGADOR_PONDERADA';
    vFactorPonderacio CATEGORIA.FACTOR_PONDERACIO%TYPE;
  BEGIN
     IBDB.PKG_UTILS.PR_AUDITAR_CANVIS(vNomObjecte, pEstadisticaPartit.ID_JUGADOR || '|' || pEstadisticaPartit.ID_PARTIT );
     -- Recuperem el factor de ponderaci de la taula de categories.
     SELECT NVL(FACTOR_PONDERACIO,0) AS FACTOR_PONDERACIO
       INTO vFactorPonderacio
       FROM IBDB.PARTIT P,
            IBDB.COMPETICIO C,
            IBDB.CATEGORIA CAT
      WHERE P.COMPETICIO_ID = C.ID
        AND C.CATEGORIA_ID = CAT.ID
        AND P.ID = pEstadisticaPartit.ID_PARTIT;
     -- Recuperem la valoraci del jugador i la multipliquem pel factor de ponderaci.
     RETURN FN_VALORACIO_JUGADOR(pEstadisticaPartit) * vFactorPonderacio;

  EXCEPTION
    WHEN OTHERS THEN
      IBDB.PKG_UTILS.PR_AUDITAR_CANVIS(vNomObjecte, SQLERRM);
      RETURN IBDB.PKG_UTILS.C_UNKNOWN_KO;
  END;

  /* Funci que permtet donar d'alta les estadstiques d'un partit assegurant la seva integritat. */
  FUNCTION FN_ALTA_ESTAD_PARTIT_JUGADOR(pEstadisticaPartit IBDB.ESTADISTICA_PARTIT%ROWTYPE) RETURN INTEGER IS
    vNomObjecte VARCHAR(31) := 'FN_ALTA_ESTAD_PARTIT_JUGADOR';
    vEstadisticaPartit IBDB.ESTADISTICA_PARTIT%ROWTYPE := pEstadisticaPartit;
  BEGIN

    IBDB.PKG_UTILS.PR_AUDITAR_CANVIS( vNomObjecte, vEstadisticaPartit.ID_JUGADOR || '|' || vEstadisticaPartit.ID_PARTIT );

    vEstadisticaPartit.MINUTS_JUGATS             := ABS(NVL(vEstadisticaPartit.MINUTS_JUGATS,0));
    vEstadisticaPartit.TIRS_LLIURES_INTENTATS    := NVL(vEstadisticaPartit.TIRS_LLIURES_INTENTATS,0);
    vEstadisticaPartit.TIRS_LLIURES_ENCISTELLATS := ABS(NVL(vEstadisticaPartit.TIRS_LLIURES_ENCISTELLATS,0));
    vEstadisticaPartit.DOS_PUNTS_INTENTATS       := NVL(vEstadisticaPartit.DOS_PUNTS_INTENTATS,0);
    vEstadisticaPartit.DOS_PUNTS_ENCISTELLATS    := ABS(NVL(vEstadisticaPartit.DOS_PUNTS_ENCISTELLATS,0));
    vEstadisticaPartit.TRES_PUNTS_INTENTATS      := NVL(vEstadisticaPartit.TRES_PUNTS_INTENTATS,0);
    vEstadisticaPartit.TRES_PUNTS_ENCISTELLATS   := ABS(NVL(vEstadisticaPartit.TRES_PUNTS_ENCISTELLATS ,0));
    vEstadisticaPartit.REBOTS_DEFENSIUS          := NVL(vEstadisticaPartit.REBOTS_DEFENSIUS,0);
    vEstadisticaPartit.REBOTS_OFENSIUS           := NVL(vEstadisticaPartit.REBOTS_OFENSIUS,0);
    vEstadisticaPartit.ASSISTENCIES              := NVL(vEstadisticaPartit.ASSISTENCIES,0);
    vEstadisticaPartit.TAPS_A_FAVOR              := NVL(vEstadisticaPartit.TAPS_A_FAVOR ,0);
    vEstadisticaPartit.TAPS_EN_CONTRA            := NVL(vEstadisticaPartit.TAPS_EN_CONTRA,0);
    vEstadisticaPartit.PILOTES_RECUPERADES       := NVL(vEstadisticaPartit.PILOTES_RECUPERADES,0);
    vEstadisticaPartit.PILOTES_PERDUDES          := NVL(vEstadisticaPartit.PILOTES_PERDUDES ,0);
    vEstadisticaPartit.FALTAS_COMESES            := NVL(vEstadisticaPartit.FALTAS_COMESES ,0);
    vEstadisticaPartit.FALTES_REBUDES            := NVL(vEstadisticaPartit.FALTES_REBUDES,0);


    vEstadisticaPartit.PUNTS := NVL(vEstadisticaPartit.TIRS_LLIURES_ENCISTELLATS,0) +
                               (NVL(vEstadisticaPartit.DOS_PUNTS_ENCISTELLATS,0)*2) +
                               (NVL(vEstadisticaPartit.TRES_PUNTS_ENCISTELLATS,0)*3);

    vEstadisticaPartit.VALORACIO_JUGADOR := FN_VALORACIO_JUGADOR(vEstadisticaPartit);
    vEstadisticaPartit.VALORACIO_JUGADOR_PONDERADA := FN_VALORACIO_JUGADOR_PONDERADA(vEstadisticaPartit);


    INSERT INTO IBDB.ESTADISTICA_PARTIT VALUES vEstadisticaPartit;
    -- Actualitzem estadistiques
    PR_ESTADISTIQUES_EST_PARTIT(pJugadorID => vEstadisticaPartit.Id_Jugador,
                                pPartitID  => vEstadisticaPartit.Id_Partit);
    COMMIT;
    RETURN IBDB.PKG_UTILS.C_OK;

  EXCEPTION
    WHEN OTHERS THEN
       IBDB.PKG_UTILS.PR_AUDITAR_CANVIS(vNomObjecte, SQLERRM);
       RETURN PKG_UTILS.C_UNKNOWN_KO;
  END FN_ALTA_ESTAD_PARTIT_JUGADOR;

  /* Funci que permtet modificar les estadstiques d'un partit assegurant la seva integritat. */
  FUNCTION FN_MODIFICA_EST_PARTIT_JUGADOR(pEstadisticaPartit IBDB.ESTADISTICA_PARTIT%ROWTYPE) RETURN INTEGER IS
    vNomObjecte VARCHAR(31) := 'FN_MODIFICA_EST_PARTIT_JUGADOR';
    vEstadisticaPartit IBDB.ESTADISTICA_PARTIT%ROWTYPE := pEstadisticaPartit;
  BEGIN

    IBDB.PKG_UTILS.PR_AUDITAR_CANVIS( vNomObjecte, vEstadisticaPartit.ID_JUGADOR || '|' || vEstadisticaPartit.ID_PARTIT );

    vEstadisticaPartit.VALORACIO_JUGADOR := FN_VALORACIO_JUGADOR(vEstadisticaPartit);
    vEstadisticaPartit.VALORACIO_JUGADOR_PONDERADA := FN_VALORACIO_JUGADOR_PONDERADA(vEstadisticaPartit);

    UPDATE IBDB.ESTADISTICA_PARTIT EP
       SET ROW = vEstadisticaPartit
     WHERE EP.ID_JUGADOR = vEstadisticaPartit.ID_JUGADOR
       AND EP.ID_PARTIT = vEstadisticaPartit.ID_PARTIT;
    -- Actualitzem estadistiques
    PR_ESTADISTIQUES_EST_PARTIT(pJugadorID => vEstadisticaPartit.Id_Jugador,
                                pPartitID  => vEstadisticaPartit.Id_Partit);
    COMMIT;
    RETURN IBDB.PKG_UTILS.C_OK;

  EXCEPTION
    WHEN OTHERS THEN
       IBDB.PKG_UTILS.PR_AUDITAR_CANVIS(vNomObjecte, SQLERRM);
       RETURN PKG_UTILS.C_UNKNOWN_KO;
  END FN_MODIFICA_EST_PARTIT_JUGADOR;

  /* Funci que permtet esborrar estadstiques d'un partit assegurant la seva integritat. */
  FUNCTION FN_BORRA_EST_PARTIT_JUGADOR(pEstadisticaPartit IBDB.ESTADISTICA_PARTIT%ROWTYPE) RETURN INTEGER IS
    vNomObjecte VARCHAR(31) := 'FN_BORRA_EST_PARTIT_JUGADOR';
    vEstadisticaPartit IBDB.ESTADISTICA_PARTIT%ROWTYPE := pEstadisticaPartit;
  BEGIN

    IBDB.PKG_UTILS.PR_AUDITAR_CANVIS( vNomObjecte, vEstadisticaPartit.ID_JUGADOR || '|' || vEstadisticaPartit.ID_PARTIT );

    DELETE FROM IBDB.ESTADISTICA_PARTIT EP
     WHERE EP.ID_JUGADOR = vEstadisticaPartit.ID_JUGADOR
       AND EP.ID_PARTIT = vEstadisticaPartit.ID_PARTIT;

    -- Actualitzem estadistiques
    PR_ESTADISTIQUES_EST_PARTIT(pJugadorID => vEstadisticaPartit.Id_Jugador,
                                pPartitID  => vEstadisticaPartit.Id_Partit);
    COMMIT;
    RETURN IBDB.PKG_UTILS.C_OK;

  EXCEPTION
    WHEN OTHERS THEN
       IBDB.PKG_UTILS.PR_AUDITAR_CANVIS( vNomObjecte, SQLERRM);
       RETURN PKG_UTILS.C_UNKNOWN_KO;
  END FN_BORRA_EST_PARTIT_JUGADOR;



END PKG_GESTIO;
/
