CREATE OR REPLACE PACKAGE IBDB.PKG_UTILS AS

  /** Paquet que conte utilitats generals de la base de dades. 
  *  
  *  @author Juanjo Daz Prez
  *  @version 1.0 Mar-Juny 2013
  **/
  
  /** Indica que tot ha anat b */
  C_OK CONSTANT INTEGER := 0;
  /** Error desconegut, normalment haurem de mirar a la taula AUDITORIA per saber quin tipus d'error es. */
  C_UNKNOWN_KO CONSTANT INTEGER := -1;
  /** Error al intentar inserir/modificar un camp amb una clau primaria ja existent. */
  C_PK_ERROR CONSTANT INTEGER := -2;
  /** Error al intentar inserir/modificar un camp amb una clau externa inexistent.*/
  C_FK_ERROR CONSTANT INTEGER := -3;
  /** La dada que s'est buscan no existeix. */
  C_NOT_FOUND CONSTANT INTEGER := -4; 
  
  
  /** Funci que ens permet auditar els canvis a la base de dades. 
  *
  *  Aquesta funci encapsula el procediment PR_AUDITAR_CANVIS afegint la funcionalitat de retornar el valor actual de la secuencia.
  *   
  *  La funci garantitza la correcta inserci dels registres de log a la taula AUDITORIA.<BR>
  *  @param pObjectName Es el nom del objecte que s'est executant.
  *  @param pCode2bSaved Es la descripci que volem enmagatzemar.
  *  @return El valor de la sequencia seq_auditoria_id o C_UNKNOWN_KO.
  *  @author Juanjo Daz Prez
  *  @version 1.0 Mar-Juny 2013      
  **/
  FUNCTION FN_AUDITAR_CANVIS(pObjectName IN VARCHAR2, pCode2bSaved IN VARCHAR2 DEFAULT NULL) RETURN INTEGER;
  
  /** Procediment que ens permet auditar els canvis a la base de dades. 
  *
  *  Aquest procediment haur de ser cridada per el desenvolupador cada cop que vulgui auditar o preparar un disparador que la cridi. 
  *  Te la particularitat de que la seva transacci es autnoma i per tant el commit que realitza en el process de auditoria no afectar al process pare que la crida. 
  *  La funci garantitza la correcta inserci dels registres de log a la taula AUDITORIA.<BR>
  *  <p>
  *  @param pObjectName es el nom del objecte que s'est executant.
  *  @param pCode2bSaved es la descripci que volem enmagatzemar.
  *  @author Juanjo Daz Prez
  *  @version 1.0 Mar-Juny 2013      
  **/  
  PROCEDURE PR_AUDITAR_CANVIS(pObjectName IN VARCHAR2, pCode2bSaved IN VARCHAR2 DEFAULT NULL);

END PKG_UTILS;
/
CREATE OR REPLACE PACKAGE BODY IBDB.PKG_UTILS AS

  /** Paquet que conte utilitats generals de la base de dades. 
  *  
  *  @author Juanjo Daz Prez
  *  @version 1.0 Mar-Juny 2013
  **/
  

  /** Funci que ens permet auditar els canvis a la base de dades. 
  *
  *  Aquesta funci encapsula el procediment PR_AUDITAR_CANVIS afegint la funcionalitat de retornar el valor actual de la secuencia.
  *   
  *  La funci garantitza la correcta inserci dels registres de log a la taula AUDITORIA.<BR>
  *  @param pObjectName Es el nom del objecte que s'est executant.
  *  @param pCode2bSaved Es la descripci que volem enmagatzemar.
  *  @return El valor de la sequencia seq_auditoria_id o C_UNKNOWN_KO.
  *  @author Juanjo Daz Prez
  *  @version 1.0 Mar-Juny 2013      
  **/   
  FUNCTION FN_AUDITAR_CANVIS(pObjectName IN VARCHAR2, pCode2bSaved IN VARCHAR2 DEFAULT NULL) RETURN INTEGER IS

  BEGIN 
    
    -- Crida a la funci d'auditoria.
    IBDB.PKG_UTILS.PR_AUDITAR_CANVIS(pObjectName, pCode2bSaved);
    -- Retornem el valor que te la sequencia desprs de l'execuci.
    RETURN IBDB.seq_auditoria_id.CURRVAL;
      
  EXCEPTION 
    WHEN OTHERS THEN
     RETURN IBDB.PKG_UTILS.C_UNKNOWN_KO;
  END FN_AUDITAR_CANVIS;
 
  /** Procediment que ens permet auditar els canvis a la base de dades. 
  *
  *  Aquest procediment haur de ser cridada per el desenvolupador cada cop que vulgui auditar o preparar un disparador que la cridi. 
  *  Te la particularitat de que la seva transacci es autnoma i per tant el commit que realitza en el process de auditoria no afectar al process pare que la crida. 
  *  La funci garantitza la correcta inserci dels registres de log a la taula AUDITORIA.<BR>
  *  <p>
  *  @param pObjectName es el nom del objecte que s'est executant.
  *  @param pCode2bSaved es la descripci que volem enmagatzemar.
  *  @author Juanjo Daz Prez
  *  @version 1.0 Mar-Juny 2013      
  **/    
  PROCEDURE PR_AUDITAR_CANVIS(pObjectName IN VARCHAR2, pCode2bSaved IN VARCHAR2 DEFAULT NULL)  IS
    PRAGMA AUTONOMOUS_TRANSACTION;
    vOwner VARCHAR2(31);
    vName VARCHAR2(31);
    vLine number;
    vCallerT VARCHAR2(100);
  BEGIN
    
    -- Treiem tota la informaci possible de l'execuci actual a partir de l'utilitat OWA_UTIL.
    OWA_UTIL.WHO_CALLED_ME (vOwner,vName,vLine,vCallerT);
    
    -- Insertem el registre afegint la sequencia manualment.
    INSERT INTO IBDB.AUDITORIA
    VALUES
    ( IBDB.seq_auditoria_id.nextval,
      user,
      sysdate,
      pObjectName,
      vOwner || '.' || vName || '->' || nvl(pCode2bSaved,'NONE') ); 
    COMMIT;  

  END PR_AUDITAR_CANVIS;

END PKG_UTILS;
/
